/*
    LBA LED flasher
    Copyright (C) 2011  Stefan Ott

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdint.h>
#include <unistd.h>

#include "blinkm_regs.h"
#include "i2c_blinkm.h"
#include "lba_blinkm.h"

extern void (*debug)(const char *, ...);

static int is_blinkm(int firmware)
{
	// 0x6163 seems to be used on newer BlinkM devices
	return (
		(firmware == BLINKM_DEVICE_FIRMWARE) ||
		(firmware == MAXM_DEVICE_FIRMWARE) ||
		(firmware == 0x6163)
	);
}

static int get_blinkm()
{
	int firmware;
	uint8_t led;

	debug("Scanning I2C bus for BlinkM devices...");

	for (led = 1; led < 128; led++) {
		firmware = blinkm_get_firmware_version(led, 0);

		if ((firmware < 1) || (firmware >= 0xffff))
			continue;

		debug("Found firmware 0x%02X at address %d", firmware, led);

		if (!is_blinkm(firmware))
			continue;
		
		debug("Valid BlinkM firmware detected at address %d", led);
		return led;
	}

	debug("No device with a valid firmware found");
	return -1;
}

static void flash(uint8_t id, int color)
{
	int r, g, b;

        r = (color & 0xff0000) >> 16;
        g = (color & 0x00ff00) >> 8;
        b = (color & 0x0000ff);

	blinkm_set_rgb_color_now(id, r, g, b);
	usleep(LBA_BLINK_LENGTH);
	blinkm_set_rgb_color_now(id, 0, 0, 0);
}

int init_blinkm()
{
	uint8_t id = get_blinkm();
	if (id == 255)
	{
		fprintf(stderr, "No BlinkM found\n");
		return -1;
	}

	debug("Initializing BlinkM at address %d", id);

	if (blinkm_stop_script(id))
	{
		// ignoring the return value since it always fails
		// perror("Failed to stop script on BlinkM")
		// return -1
	}
	return 0;
}

void show_status(uint8_t id, int sys, int bt, int wifi)
{

	flash(id, sys);
	usleep(LBA_BLINK_INTERVAL);
	flash(id, bt);
	usleep(LBA_BLINK_INTERVAL);
	flash(id, wifi);
}
